import wx
from functools import partial
from .utils import LEFT, CEN

class SimpleText(wx.StaticText):
    "simple static text wrapper"
    def __init__(self, parent, label, minsize=None, font=None, colour=None,
                 bgcolour=None, style=CEN, tooltip=None, **kws):
        wx.StaticText.__init__(self, parent, -1, label=label, style=style,
                               **kws)
        # if tooltip is not None:
        #     self.SetTooltip(tooltip)
        if minsize is not None:
            self.SetMinSize(minsize)
        if font is not None:
            self.SetFont(font)
        if colour is not None:
            self.SetForegroundColour(colour)
        if bgcolour is not None:
            self.SetBackgroundColour(bgcolour)

class TextCtrl(wx.TextCtrl):
    """simple TextCtrl
    t = TextCtrl(parent, value, font=None
                 colour=None, bgcolour=None,
                 action=None, action_kws=None,
                 act_on_losefocus=True, **kws)
    has a method SetAction(action, action_kws)
    for setting action on RETURN (and optionally LoseFocus)
    """
    def __init__(self, parent, value, font=None,
                 colour=None, bgcolour=None,
                 action=None, action_kws=None,
                 act_on_losefocus=True, tooltip=None, **kws):

        self.act_on_losefocus = act_on_losefocus

        this_sty =   wx.TE_PROCESS_ENTER
        if 'style' in kws:
            this_sty = this_sty | kws['style']
        kws['style'] = this_sty

        wx.TextCtrl.__init__(self, parent, -1, **kws)
        self.SetValue(value)
        if font is not None:
            self.SetFont(font)
        if colour is not None:
            self.SetForegroundColour(colour)
        if bgcolour is not None:
            self.SetBackgroundColour(bgcolour)
        if tooltip is not None:
            self.SetTooltip(tooltip)
        if action_kws is None:
            action_kws = {}

        self.SetAction(action, **action_kws)

        self.Bind(wx.EVT_CHAR, self.onChar)
        self.Bind(wx.EVT_KILL_FOCUS, self.onFocus)

    def SetAction(self, action, **kws):
        "set action callback"
        self.__act = None
        if callable(action):
            self.__act = partial(action, **kws)

    def onFocus(self, evt=None):
        "focus events -- may act on KillFocus"
        if self.act_on_losefocus and self.__act is not None:
            self.__act(self.GetValue())
        evt.Skip()

    def onChar(self, evt=None):
        "character events -- may act on RETURN"
        if evt.GetKeyCode() == wx.WXK_RETURN and self.__act is not None:
            self.__act(self.GetValue())
        evt.Skip()


class LabeledTextCtrl(TextCtrl):
    """
    simple extension of TextCtrl with a .label attribute holding a SimpleText
    Typical usage:
      entry = LabeledTextCtrl(self, value='22', labeltext='X:')
      row   = wx.BoxSizer(wx.HORIZONTAL)
      row.Add(entry.label, 1,wx.ALIGN_LEFT|wx.EXPAND)
      row.Add(entry,    1,wx.ALIGN_LEFT|wx.EXPAND)
    """
    def __init__(self, parent, value, font=None, action=None,
                 action_kws=None, act_on_losefocus=True, size=(-1, -1),
                 bgcolour=None, colour=None, style=LEFT,
                 labeltext=None, labelsize=(-1, -1),
                 labelcolour=None, labelbgcolour=None, **kws):

        if labeltext is not None:
            self.label = SimpleText(parent, labeltext, size=labelsize,
                                    font=font, style=style,
                                    colour=labelcolour,
                                    bgcolour=labelbgcolour)

        try:
            value = str(value)
        except:
            value = ' '

        TextCtrl.__init__(self, parent, value, font=font,
                          colour=colour, bgcolour=bgcolour,
                          style=style, size=size,
                          action=action, action_kws=action_kws,
                          act_on_losefocus=act_on_losefocus, **kws)


class HyperText(wx.StaticText):
    """HyperText is a simple extension of wx.StaticText that

       1. adds an underscore to the label to appear to be a hyperlink
       2. performs the supplied action on Left-Up button events
    """

    def __init__(self, parent, label, action=None, colour=(50, 50, 180),
                  bgcolour=None, underline=True, **kws):
        wx.StaticText.__init__(self, parent, -1, label=label, **kws)
        self.SetForegroundColour(colour)
        if bgcolour is not None:
            self.SetBackgroundColour(bgcolour)
        if underline:
            font  = self.GetFont()
            try:
                font.SetUnderlined(True)
            except:
                pass
            self.SetFont(font)
        self.action = action
        self.Bind(wx.EVT_LEFT_UP, self.OnSelect)

    def OnSelect(self, event=None):
        "Left-Up Event Handler"
        if hasattr(self.action,'__call__'):
            self.action(label=self.GetLabel(), event=event)
        event.Skip()



import wx

def Button(parent, label, action=None, **kws):
    """Simple button with bound action
    b = Button(parent, label, action=None, **kws)

    """
    thisb = wx.Button(parent, label=label, **kws)
    if callable(action):
        parent.Bind(wx.EVT_BUTTON, action, thisb)
    return thisb

def BitmapButton(parent, bmp, action=None, tooltip=None, size=(20, 20), **kws):
    b = wx.BitmapButton(parent, id=-1, bitmap=bmp, size=size, **kws)
    if action is not None:
        parent.Bind(wx.EVT_BUTTON, action, b)
    if tooltip is not None:
        b.SetToolTip(tooltip)
    return b

def ToggleButton(parent, label, action=None, tooltip=None,
                 size=(25, 25), **kws):
    b = wx.ToggleButton(parent, -1, label, size=size, **kws)
    if action is not None:
        b.Bind(wx.EVT_TOGGLEBUTTON, action)
    if tooltip is not None:
        b.SetToolTip(tooltip)
    return b



import wx

class EditableListBox(wx.ListBox):
    """
    A ListBox with pop-up menu to arrange order of
    items and remove items from list
    supply select_action for EVT_LISTBOX selection action
    """

    def __init__(self, parent, select_action, right_click=True,
                 remove_action=None, color=(10, 10, 20),
                 bgcolor=(240, 240, 230), **kws):

        wx.ListBox.__init__(self, parent, **kws)

        if isinstance(bgcolor, (tuple, list)):
            bgcolor = wx.Colour(bgcolor)
        if isinstance(color, (tuple, list)):
            color = wx.Colour(color)

        self.SetBackgroundColour(bgcolor)
        self.SetOwnBackgroundColour(bgcolor)
        self.SetForegroundColour(color)
        self.SetOwnForegroundColour(color)

        self.Bind(wx.EVT_LISTBOX,  select_action)
        self.remove_action = remove_action
        if right_click:
            self.Bind(wx.EVT_RIGHT_DOWN, self.onRightClick)

    def onRightClick(self, evt=None):
        menu = wx.Menu()
        self.pmenu_labels = {}
        for menulabel in ('Move up', 'Move down', 'Move to top',
                          'Move to bottom', 'Remove from list'):
            item = menu.Append(-1, menulabel)
            self.pmenu_labels[item.Id] = menulabel
            self.Bind(wx.EVT_MENU, self.onRightEvent, item)
        self.PopupMenu(menu)
        menu.Destroy()

    def onRightEvent(self, event=None):
        idx = self.GetSelection()
        mlabel = self.pmenu_labels.get(event.GetId(), None)
        if idx < 0 or mlabel is None:
            return

        names = self.GetItems()
        this  = names.pop(idx)

        if mlabel == 'Move up' and idx > 0:
            names.insert(idx-1, this)
        elif mlabel == 'Move down' and idx < len(names):
            names.insert(idx+1, this)
        elif mlabel == 'Move to top':
            names.insert(0, this)
        elif mlabel == 'Move to bottom':
            names.append(this)
        elif mlabel == 'Remove from list' and self.remove_action is not None:
            self.remove_action(this)

        self.Clear()
        for name in names:
            self.Append(name)



#!/usr/bin/env python
"""
Code for floating point controls
"""
import sys
from functools import partial
import wx
from wx.lib.agw import floatspin as fspin

from .icons import get_icon

HAS_NUMPY = False
try:
    import numpy
    HAS_NUMPY = True
except ImportError:
    pass


def make_steps(prec=3, tmin=0, tmax=10, base=10, steps=(1, 2, 5)):
    """make a list of 'steps' to use for a numeric ComboBox
    returns a list of floats, such as
        [0.01, 0.02, 0.05, 0.10, 0.20, 0.50, 1.00, 2.00...]
    """
    steplist = []
    power = -prec
    step = tmin
    while True:
        decade = base**power
        for step in (j*decade for j in steps):
            if step > 0.99*tmin and step <= tmax and step not in steplist:
                steplist.append(step)
        if step >= tmax:
            break
        power += 1
    return steplist

def set_float(val, default=0):
    """ utility to set a floating value,
    useful for converting from strings """
    out = None
    if not val in (None, ''):
        try:
            out = float(val)
        except ValueError:
            return None
        if HAS_NUMPY:
            if numpy.isnan(out):
                out = default
        else:
            if not(out > 0) and not(out<0) and not(out==0):
                out = default
    return out

class FloatCtrl(wx.TextCtrl):
    """ Numerical Float Control::
    wx.TextCtrl that allows only numerical input

    takes a precision argument
    and optional upper / lower bounds
    options:
     action             callback on Enter (or lose focus)
     act_on_losefocus   boolean (default False) to act on lose focus events
     action_kws         keyword args for action

    """
    def __init__(self, parent, value='', minval=None, maxval=None,
                 precision=3, odd_only=False, even_only=False,
                 bell_on_invalid = True,
                 act_on_losefocus=False, gformat=False,
                 action=None, action_kws=None, **kws):

        self.__digits = '0123456789.-e'
        self.__prec   = precision
        self.odd_only = odd_only
        self.even_only = even_only
        if precision is None:
            self.__prec = 0
        self.format   = '%%.%if' % self.__prec
        self.gformat  = gformat
        if gformat:
            self.__prec = max(8, self.__prec)
            self.format = '%%.%ig' % self.__prec

        self.is_valid = True
        self.__val = set_float(value)
        self.__max = set_float(maxval)
        self.__min = set_float(minval)
        self.__bound_val = None
        self.__mark = None
        self.__action = None

        self.fgcol_valid   = "Black"
        self.bgcol_valid   = "White"
        self.fgcol_invalid = "Red"
        self.bgcol_invalid = (254, 254, 80)
        self.bell_on_invalid = bell_on_invalid
        self.act_on_losefocus = act_on_losefocus

        # set up action
        if action_kws is None:
            action_kws = {}
        self.SetAction(action, **action_kws)

        this_sty =  wx.TE_PROCESS_ENTER|wx.TE_RIGHT
        if 'style' in kws:
            this_sty = this_sty | kws['style']
        kws['style'] = this_sty

        wx.TextCtrl.__init__(self, parent, wx.ID_ANY, **kws)

        self.__CheckValid(self.__val)
        self.SetValue(self.__val)

        self.Bind(wx.EVT_CHAR, self.OnChar)
        self.Bind(wx.EVT_TEXT, self.OnText)

        self.Bind(wx.EVT_SET_FOCUS,  self.OnSetFocus)
        self.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)
        self.__GetMark()

    def SetAction(self, action, **kws):
        "set callback action"
        if hasattr(action,'__call__'):
            self.__action = partial(action, **kws)

    def SetPrecision(self, prec=0, gformat=None):
        "set precision"
        self.__prec = prec
        if gformat is not None:
            self.gformat = gformat
        if self.gformat:
            self.__prec = max(8, self.__prec)
            self.format = '%%.%ig' % self.__prec
        else:
            self.format = '%%.%if' % prec

    def __GetMark(self):
        " keep track of cursor position within text"
        try:
            self.__mark = min(wx.TextCtrl.GetSelection(self)[0],
                              len(wx.TextCtrl.GetValue(self).strip()))
        except:
            self.__mark = 0

    def __SetMark(self, mark=None):
        "set mark for later"
        if mark is None:
            mark = self.__mark
        self.SetSelection(mark, mark)

    def SetValue(self, value=None, act=True):
        " main method to set value "
        if value is None:
            value = wx.TextCtrl.GetValue(self).strip()
        self.__CheckValid(value)
        self.__GetMark()
        value = set_float(value)
        if value is not None:
            wx.TextCtrl.SetValue(self, self.format % value)

        if self.is_valid and hasattr(self.__action, '__call__') and act:
            self.__action(value=self.__val)
        elif not self.is_valid and self.bell_on_invalid:
            wx.Bell()

        self.__SetMark()

    def OnKillFocus(self, event):
        "focus lost"
        self.__GetMark()
        if self.act_on_losefocus and hasattr(self.__action, '__call__'):
            self.__action(value=self.__val)
        event.Skip()

    def OnSetFocus(self, event):
        "focus gained - resume editing from last mark point"
        self.__SetMark()
        event.Skip()

    def OnChar(self, event):
        """ on Character event"""
        key   = event.GetKeyCode()
        entry = wx.TextCtrl.GetValue(self).strip()
        pos   = wx.TextCtrl.GetSelection(self)[0]
        # really, the order here is important:
        # 1. return sends to ValidateEntry
        if key == wx.WXK_RETURN:
            if not self.is_valid:
                wx.TextCtrl.SetValue(self, self.format % set_float(self.__bound_val))
            else:
                self.SetValue(entry)
            return

        # 2. other non-text characters are passed without change
        if (key < wx.WXK_SPACE or key == wx.WXK_DELETE or key > 255):
            event.Skip()
            return

        # 3. check for multiple '.' and out of place '-' signs and ignore these
        #    note that chr(key) will now work due to return at #2
        ckey = chr(key)
        if ((ckey == '.' and (self.__prec == 0 or '.' in entry)) or
            (ckey != '-' and pos == 0 and entry.startswith('-'))):
            return
        if 'e' in entry:
            if ckey == 'e':
                return
            ind_e = entry.index('e')
            if (ckey == '-' and pos not in (0, ind_e+1)):
                return
        elif (ckey == '-' and '-' in entry):
            return
        # allow 'inf' and '-inf', but very restricted
        if ckey in ('i', 'n', 'f'):
            has_num = any([x in entry for x in self.__digits[:11]])
            if ((ckey in entry) or has_num or
                entry not in ('', '-', '-i', 'i', '-n', 'n', '-f', 'f',
                              '-in', 'in', '-if', 'if',
                             '-nf', 'nf', '-inf', 'inf')):
                return
        elif (ckey != '-' and ckey in self.__digits and
              any([x in ('i', 'n', 'f') for x in entry])):
            return

        # 4. allow digits or +/-inf but not other characters
        if ckey in self.__digits or ckey in ('i', 'n', 'f'):
            event.Skip()

    def OnText(self, event=None):
        "text event"
        try:
            if event.GetString() != '':
                self.__CheckValid(event.GetString())
        except:
            pass
        event.Skip()

    def GetValue(self):
        if self.__prec > 0:
            return set_float("%%.%if" % (self.__prec) % (self.__val))
        else:
            return int(self.__val)

    def GetMin(self):
        "return min value"
        return self.__min

    def GetMax(self):
        "return max value"
        return self.__max

    def SetMin(self, val):
        "set min value"
        self.__min = set_float(val)

    def SetMax(self, val):
        "set max value"
        self.__max = set_float(val)

    def __CheckValid(self, value):
        "check for validity of value"
        val = self.__val
        self.is_valid = True
        try:
            val = set_float(value)
            if self.__min is not None and (val < self.__min):
                self.is_valid = False
                val = self.__min
            if self.__max is not None and (val > self.__max):
                self.is_valid = False
                val = self.__max
            if self.__prec == 0:
                if self.odd_only and (val % 2 == 0):
                    self.is_valid = False
                    val = val + 1
                elif self.even_only and (val %2 == 1):
                    self.is_valid = False
                    val = val + 1
        except:
            self.is_valid = False
        self.__bound_val = self.__val = val
        fgcol, bgcol = self.fgcol_valid, self.bgcol_valid
        if not self.is_valid:
            fgcol, bgcol = self.fgcol_invalid, self.bgcol_invalid

        self.SetForegroundColour(fgcol)
        self.SetBackgroundColour(bgcol)
        self.Refresh()


def FloatSpin(parent, value=0, action=None, tooltip=None, size=(100, -1),
              digits=1, increment=1, use_gtk3=False, **kws):
    """FloatSpin with action and tooltip"""
    if value is None:
        value = 0

    # need to work this out better for GTK3 - lots of small
    # differences with GTK2, but this one is the biggest headache.
    # SpinCtrlDouble is like FloatSpin, but with every option
    # having a slightly different name...
    if use_gtk3 and 'gtk3' in wx.PlatformInfo:
        maxval = kws.pop('max_val', None)
        minval = kws.pop('min_val', None)
        fmt = "%%%df" % digits
        fs = wx.SpinCtrlDouble(parent, -1, value=fmt % value,
                               size=(size[0]+40, size[1]),
                               inc=increment, **kws)
        fs.SetDigits(digits)
        if minval is not None:
            fs.SetMin(minval)
        if maxval is not None:
            fs.SetMax(maxval)

        if action is not None:
            fs.Bind(wx.EVT_SPINCTRLDOUBLE, action)

    else:
        fs = fspin.FloatSpin(parent, -1, size=size, value=value,
                             digits=digits, increment=increment, **kws)

        if action is not None:
            fs.Bind(fspin.EVT_FLOATSPIN, action)
    if tooltip is not None:
        fs.SetToolTip(tooltip)
    return fs


def FloatSpinWithPin(parent, value=0, pin_action=None,
                     tooltip='select point from plot', **kws):
    """create a FloatSpin with Pin button with action"""
    fspin = FloatSpin(parent, value=value, **kws)
    bmbtn = wx.BitmapButton(parent, id=-1, bitmap=get_icon('pin'),
                            size=(25, 25))
    if pin_action is not None:
        parent.Bind(wx.EVT_BUTTON, pin_action, bmbtn)
    bmbtn.SetToolTip(tooltip)
    return fspin, bmbtn

class NumericCombo(wx.ComboBox):
    """
    Numeric Combo: ComboBox with numeric-only choices
    """
    def __init__(self, parent, choices, precision=3, fmt=None,
                 init=0, default_val=None, width=80):

        self.fmt = fmt
        if fmt is None:
            self.fmt = "%%.%if" % precision

        self.choices  = choices
        schoices = [self.fmt % i for i in self.choices]
        wx.ComboBox.__init__(self, parent, -1, '', (-1, -1), (width, -1),
                             schoices, wx.CB_DROPDOWN|wx.TE_PROCESS_ENTER)

        init = min(init, len(self.choices))
        if default_val is not None:
            if default_val in schoices:
                self.SetStringSelection(default_val)
            else:
                self.add_choice(default_val, select=True)
        else:
            self.SetStringSelection(schoices[init])
        self.Bind(wx.EVT_TEXT_ENTER, self.OnEnter)

    def OnEnter(self, event=None):
        self.add_choice(float(event.GetString()))

    def add_choice(self, val, select=True):
        if val not in self.choices:
            self.choices.append(val)
            self.choices.sort()
        self.choices.reverse()
        self.Clear()
        self.AppendItems([self.fmt % x for x in self.choices])
        if select:
            self.SetSelection(self.choices.index(val))



import time
import os
import sys
import wx
from functools import partial

from wxutils import (Button, CEN, Check, Choice, EditableListBox, OkCancel,
                     FRAMESTYLE, FileOpen, FileSave, FloatCtrl, FloatSpin,
                     Font, GUIColors, GridPanel, LabeledTextCtrl, HLine,
                     HyperText, LEFT, MenuItem, Popup, RIGHT, RowPanel,
                     SimpleText, TextCtrl, fix_filename, get_icon, pack,
                     BitmapButton, ToggleButton, YesNo, NumericCombo,
                     make_steps)
from wxutils.periodictable import PeriodicTablePanel, PTableFrame
from wxutils.filechecklist import FileCheckList

PY_FILES = "Python scripts (*.py)|*.py"
ALL_FILES = "All files (*.*)|*.*"

class DemoFrame(wx.Frame):
    def __init__(self, idletime=30.0):

        wx.Frame.__init__(self, None, -1, 'wxutil demo',
                          style=wx.DEFAULT_FRAME_STYLE|wx.RESIZE_BORDER|wx.TAB_TRAVERSAL)
        self.SetTitle('wxutil demo')

        self.SetFont(Font(11))
        self.idletime = idletime
        self.deadtime = time.time() + idletime

        self.set_menu()
        self.statusbar = self.CreateStatusBar(2, 1)
        self.statusbar.SetStatusWidths([-2, -1])
        statusbar_fields = ['Initializing....', ' ']
        for i in range(len(statusbar_fields)):
            self.statusbar.SetStatusText(statusbar_fields[i], i)

        self.Bind(wx.EVT_CLOSE, self.onExit)

        panel  = GridPanel(self, nrows=8, ncols=4)

        tctrl_name = TextCtrl(panel, value='', action=self.onName,
                              size=(250, -1))

        lctrl_addr = LabeledTextCtrl(panel, value='<>', action=self.onAddr,
                                     labeltext=' Address: ', size=(250, -1))

        lab3 = HyperText(panel,' FloatCtrl: ', size=(100, -1), action=self.onHyperText)

        val3 = FloatCtrl(panel, '3', action=self.onFloat1, precision=2,
                         minval=0, maxval=1000, size=(250, -1))


        lab4 = HyperText(panel,' FloatSpin: ', size=(100, -1), action=self.onHyperText)
        val4 = FloatSpin(panel, '12.2', action=self.onFloatSpin, digits=2,
                         increment=0.1, size=(250, -1))

        labx = HyperText(panel,' NumericCombo: ', size=(100, -1), action=self.onHyperText)

        steps = make_steps(prec=1, tmin=0, tmax=100)
        valx = NumericCombo(panel, steps, precision=1)

        self.choice1 = Choice(panel, size=(200, -1),action=self.onChoice)
        self.choice1.SetChoices(['Apple', 'Banana', 'Cherry'])

        yesno = YesNo(panel)

        check1 = Check(panel, label='enable? ',   action=self.onCheck)

        btn1 = Button(panel, label='Start', size=(100, -1), action=self.onStart)

        pinbtn = BitmapButton(panel, get_icon('pin'), size=(50, -1),
                              action=partial(self.onBMButton, opt='pin1'),
                              tooltip='use last point selected from plot')

        togbtn = ToggleButton(panel, 'Press Me',
                              action=self.onToggleButton, size=(100, -1),
                              tooltip='do it, do it now, you will like it')


        browse_btn = Button(panel, 'Open File',
                            action=self.onFileOpen, size=(150, -1))


        okcancel = OkCancel(panel, onOK=self.onOK, onCancel=self.onCancel)

        ptable_btn = Button(panel, 'Show Periodic Table',
                            action=self.onPTable, size=(175, -1))


        edlist_btn = Button(panel, 'Show Editable Listbox',
                            action=self.onEdList, size=(175, -1))

        filelist_btn = Button(panel, 'Show File CheckList',
                            action=self.onFileList, size=(175, -1))


        panel.AddText(' Name: ', style=LEFT)

        panel.Add(tctrl_name, dcol=2)
        panel.Add(lctrl_addr.label, newrow=True)
        panel.Add(lctrl_addr, dcol=2)

        panel.Add(lab3, newrow=True)
        panel.Add(val3, dcol=3)

        panel.Add(lab4, newrow=True)
        panel.Add(val4, dcol=3)

        panel.Add(labx, newrow=True)
        panel.Add(valx, dcol=3)

        panel.AddText(' Choice : ', newrow=True)
        panel.Add(check1)
        panel.Add(self.choice1)
        panel.AddText(' Yes or No: ', newrow=True)
        panel.Add(yesno)
        panel.Add(HLine(panel, size=(500, -1)), dcol=3, newrow=True)

        panel.Add(btn1, newrow=True)
        panel.Add(pinbtn)
        panel.Add(togbtn)

        panel.Add(browse_btn, newrow=True)
        panel.Add(ptable_btn)
        panel.Add(edlist_btn, newrow=True)
        panel.Add(filelist_btn)

        panel.Add(okcancel, newrow=True)
        panel.pack()

        self.timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.onTimer, self.timer)

        fsizer = wx.BoxSizer(wx.VERTICAL)
        fsizer.Add(panel, 0, LEFT|wx.EXPAND)
        wx.CallAfter(self.init_timer)

        psize = panel.GetBestSize()
        self.SetSize((psize[0]+5, psize[1]+25))

        pack(self, fsizer)
        self.Refresh()

    def init_timer(self):
        self.timer.Start(100)

    def onTimer(self, event=None):
        time_remaining = self.deadtime  - time.time()

        msg = "Time remaining = %.1f sec" % (time_remaining)
        self.statusbar.SetStatusText(msg, 1)
        if time_remaining < 0:
            print("quitting...")
            self.onExit()

    def report(self, reason, value):
        self.statusbar.SetStatusText("%s: %s" % (reason, value), 0)
        self.deadtime = time.time() + self.idletime

    def set_menu(self):
        menu = wx.Menu()
        mexit = MenuItem(self, menu, "Q&uit", "Quit Program", self.onExit)

        menubar = wx.MenuBar()
        menubar.Append(menu, "&File");
        self.SetMenuBar(menubar)

    def onOK(self, event=None):
        self.report("on OK: ", '')

    def onCancel(self, event=None):
        self.report("on Cancel: ", '')

    def onName(self, event=None):
        self.report("on Name: ", event)

    def onAddr(self, event=None):
        self.report("on Addr: ", event)

    def onHyperText(self, event=None, label=None):
        self.report("on HyperText ", label)

    def onFloat1(self, value=None):
        self.report("on Float1 ", value)

    def onFloatSpin(self, event=None):
        self.report("on FloatSpin ", event.GetString())

    def onChoice(self, event=None):
        self.report("on Choice ", event.GetString())

    def onCheck(self, event=None):
        self.report("on Check ", event.IsChecked())

        self.choice1.Enable(event.IsChecked())

    def onStart(self, event=None):
        self.report("on Start Button ", '')

    def onBMButton(self, event=None, opt='xxx'):
        self.report("on Bitmap Button ", opt)

    def onToggleButton(self, event=None):
        self.report(" on Toggle Button %s " % event.GetString(), event.IsChecked())

    def onPTable(self, event=None):
        PTableFrame(fontsize=10).Show()
        self.report("on Periodic Table: ", '')

    def onEdList(self, event=None):
        frame = wx.Frame(self)
        edlist  = EditableListBox(frame, self.onEdListSelect)
        edlist.Append(" Item 1 ")
        edlist.Append(" Item 2 ")
        edlist.Append(" Next ")
        self.report("on Edit List: ", '')
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(edlist, 1, wx.EXPAND|wx.ALL, 5)
        pack(frame, sizer)

        frame.Show()
        frame.Raise()

    def onEdListSelect(self, event=None, **kws):
        self.report(" Editable List selected ", event.GetString())

    def onFileList(self, event=None):
        frame = wx.Frame(self)
        edlist  = FileCheckList(frame, select_action=self.onFileListSelect)
        for i in range(8):
            edlist.Append("File.%3.3d" % (i+1))
        self.report("on File List: ", '')
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(edlist, 1, wx.EXPAND|wx.ALL, 5)
        pack(frame, sizer)

        frame.Show()
        frame.Raise()

    def onFileListSelect(self, event=None, **kws):
        self.report(" File List selected ", event.GetString())

    def onFileOpen(self, event=None):
        wildcards = "%s|%s" % (PY_FILES, ALL_FILES)

        dlg = wx.FileDialog(self, message='Select File',
                            wildcard=wildcards,
                            style=wx.FD_OPEN)

        if dlg.ShowModal() == wx.ID_OK:
            path = os.path.abspath(dlg.GetPath())
            self.report("file ",  path)
        dlg.Destroy()

    def onExit(self, event=None):
        self.Destroy()


def test_wxdemo(idletime=5.0):
    app = wx.App()
    DemoFrame(idletime=idletime).Show(True)
    app.MainLoop()

if __name__ == '__main__':
    test_wxdemo(idletime=30.0)



import wx
import time
import unittest
import pytest

from wxutils import (Button, CEN, Check, Choice, EditableListBox,
                     FRAMESTYLE, FileOpen, FileSave, FloatCtrl, Font,
                     GUIColors, GridPanel, HLine, HyperText, LCEN, LEFT,
                     MenuItem, Popup, RCEN, RIGHT, RowPanel, SimpleText,
                     TextCtrl, fix_filename, get_icon, pack)

class TestCase(unittest.TestCase):
    def test_imports(self):
        import wxutils

        assert('Check' in dir(wxutils))

if __name__ == '__main__':
    pytest.main(['-v', '-x', '-s'])



#!/usr/bin/env python
# epics/wx/utils.py
"""
date utilities and widgets
"""
import os
from datetime import timedelta

import wx
import wx.lib.masked as masked

def hms(secs):
    "format time in seconds to H:M:S"
    return str(timedelta(seconds=int(secs)))

class DateTimeCtrl(object):
    """
    Simple Combined date/time control
    """
    def __init__(self, parent, name='datetimectrl', use_now=False):
        self.name = name
        panel = self.panel = wx.Panel(parent)
        bgcol = wx.Colour(250, 250, 250)

        datestyle = wx.DP_DROPDOWN|wx.DP_SHOWCENTURY|wx.DP_ALLOWNONE

        self.datectrl = wx.DatePickerCtrl(panel, size=(120, -1),
                                          style=datestyle)
        self.timectrl = masked.TimeCtrl(panel, -1, name=name,
                                        limited=False,
                                        fmt24hr=True, oob_color=bgcol)
        timerheight = self.timectrl.GetSize().height
        spinner = wx.SpinButton(panel, -1, wx.DefaultPosition,
                                (-1, timerheight), wx.SP_VERTICAL )
        self.timectrl.BindSpinButton(spinner)
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.datectrl, 0, wx.ALIGN_CENTER)
        sizer.Add(self.timectrl, 0, wx.ALIGN_CENTER)
        sizer.Add(spinner, 0,  wx.ALIGN_LEFT)
        panel.SetSizer(sizer)
        sizer.Fit(panel)
        if use_now:
            self.timectrl.SetValue(wx.DateTime_Now())



#!/usr/bin/env python
"""
file utilities
"""
import os
import sys
import time
import wx
from collections import namedtuple

from .text import SimpleText
from .gridpanel import GridPanel
from .paths import get_configfile, save_configfile, get_cwd

def fix_filename(s):
    """fix string to be a 'good' filename.
    This may be a more restrictive than the OS, but
    avoids nasty cases."""
    badchars = ' <>:"\'\\\t\r\n/|?*!%$'
    t = s.translate(s.maketrans(badchars, '_'*len(badchars)))
    if t.count('.') > 1:
        for i in range(t.count('.') - 1):
            idot = t.find('.')
            t = "%s_%s" % (t[:idot], t[idot+1:])
    return t



def FileOpen(parent, message, default_dir=None, default_file=None,
             multiple=False, wildcard=None):
    """File Open dialog wrapper.
    returns full path on OK or None on Cancel
    """
    out = None
    if default_dir is None:
        try:
            default_dir = get_cwd()
        except:
            pass
    if wildcard is None:
        wildcard = 'All files (*.*)|*.*'

    style = wx.FD_OPEN|wx.FD_CHANGE_DIR
    if multiple:
        style = style|wx.FD_MULTIPLE
    dlg = wx.FileDialog(parent, message=message, wildcard=wildcard,
                        defaultFile=default_file,
                        defaultDir=default_dir,
                        style=style)

    out = None
    if dlg.ShowModal() == wx.ID_OK:
        out = os.path.abspath(dlg.GetPath())
    dlg.Destroy()
    return out

def FileSave(parent, message, default_file=None,
             default_dir=None,  wildcard=None):
    "File Save dialog"
    out = None
    if wildcard is None:
        wildcard = 'All files (*.*)|*.*'

    if default_dir is None:
        try:
            default_dir = get_cwd()
        except:
            pass

    dlg = wx.FileDialog(parent, message=message, wildcard=wildcard,
                        defaultFile=default_file,
                        style=wx.FD_SAVE|wx.FD_CHANGE_DIR)
    if dlg.ShowModal() == wx.ID_OK:
        out = os.path.abspath(dlg.GetPath())
    dlg.Destroy()
    return out

def SelectWorkdir(parent,  message='Select Working Folder...'):
    "prompt for and change into a working directory "
    dlg = wx.DirDialog(parent, message,
                       style=wx.DD_DEFAULT_STYLE|wx.DD_CHANGE_DIR)

    path = os.path.abspath(os.curdir)
    dlg.SetPath(path)
    if  dlg.ShowModal() == wx.ID_CANCEL:
        return None
    path = os.path.abspath(dlg.GetPath())
    dlg.Destroy()
    os.chdir(path)
    return path


def OkCancel(panel, onOK=None, onCancel=None):
    """Add OK / Cancel buttons
    OkCancel(panel, onOK=None, onCancel=None)
    returns a wx.StdDialogButtonSizer
    """
    btnsizer = wx.StdDialogButtonSizer()
    _ok = wx.Button(panel, wx.ID_OK)
    _no = wx.Button(panel, wx.ID_CANCEL)
    panel.Bind(wx.EVT_BUTTON, onOK,     _ok)
    panel.Bind(wx.EVT_BUTTON, onCancel, _no)
    _ok.SetDefault()
    btnsizer.AddButton(_ok)
    btnsizer.AddButton(_no)
    btnsizer.Realize()
    return btnsizer

class SavedParameterDialog(wx.Dialog):
    """dialog to prompt for string with
    read/write of previous value to config file

    dlg = SavedParameterDialog(label='Prefix',
                               title='Parameter Prompt',
                               configfile='.myparam.dat')
    res = dlg.GetResponse()
    dlg.Destroy()
    if res.ok:
        prefix = res.value

    """
    def __init__(self, label='Value:', title='Parameter Prompt',
                 configfile=None, **kws):
        wx.Dialog.__init__(self, None, wx.ID_ANY, title=title)

        value = ''
        self.configfile = configfile
        if configfile is not None:
            cfile = get_configfile(configfile)
            if cfile is not None:
                with open(cfile, 'r') as fh:
                    for line in fh.readlines():
                        line = line.strip().replace('\n','').replace('\r','')
                        if line.startswith('#') or len(line)<1:
                            continue
                        value = line
                        break


        self.text = wx.TextCtrl(self, value=value, size=(250, -1))
        panel = GridPanel(self, ncols=3, nrows=4, pad=2)
        panel.Add(wx.StaticText(self, label=label), newrow=True)
        panel.Add(self.text)
        panel.Add(OkCancel(self), dcol=2, newrow=True)
        panel.pack()

    def GetResponse(self, master=None, gname=None, ynorm=True):
        self.Raise()
        response = namedtuple('Reponse', ('ok', 'value'))
        ok, value = False, None
        if self.ShowModal() == wx.ID_OK:
            value = self.text.GetValue()
            ok = True
        if ok and self.configfile is not None:
            tout = '# saved %s\n  %s   \n\n' % (time.ctime(), value)
            save_configfile(self.configfile, tout)
        return response(ok, value)



#!/usr/bin/env python

"""
A collection of wx utility functions,
mostly simplified wrappers around existing widgets.
"""
import os
import sys
from traceback import format_tb
import wx
from  wx.lib.dialogs import ScrolledMessageDialog

# some common abbrevs for wx ALIGNMENT styles
RIGHT = RCEN = wx.ALIGN_RIGHT
LEFT  = LCEN = wx.ALIGN_LEFT
CEN   = CCEN = wx.ALIGN_CENTER
LTEXT = wx.ST_NO_AUTORESIZE|wx.ALIGN_CENTER
FRAMESTYLE = wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL

def SetTip(wid, tip=''):
    wid.SetToolTip(tip)

def set_sizer(panel, sizer=None, style=wx.VERTICAL, fit=False):
    """ utility for setting wx Sizer  """
    if sizer is None:
        sizer = wx.BoxSizer(style)
    panel.SetAutoLayout(1)
    panel.SetSizer(sizer)
    if fit:
        sizer.Fit(panel)

def pack(window, sizer, expand=1.1):
    "simple wxPython pack function"
    tsize =  window.GetSize()
    msize =  window.GetMinSize()
    window.SetSizer(sizer)
    sizer.Fit(window)
    nsize = (10*int(expand*(max(msize[0], tsize[0])/10)),
             10*int(expand*(max(msize[1], tsize[1])/10.)))
    window.SetSize(nsize)

def Font(size, serif=False):
    """define a font by size and serif/ non-serif
    f = Font(10, serif=True)
    """
    family = wx.DEFAULT
    if not serif:
        family = wx.SWISS
    return wx.Font(size, family, wx.NORMAL, wx.BOLD, 0, "")

def SetChildrenFont(widget, font, dsize=None):
    "set font for a widget and all children"
    cfont = widget.GetFont()
    font.SetWeight(cfont.GetWeight())
    if dsize == None:
        dsize = font.PointSize - cfont.PointSize
    else:
        font.PointSize = cfont.PointSize + dsize
    widget.SetFont(font)
    for child in widget.GetChildren():
        set_font_with_children(child, font, dsize=dsize)

def HLine(parent, size=(700, 3)):
    """Simple horizontal line
    h = HLine(parent, size=(700, 3)
    """
    return wx.StaticLine(parent, size=size, style=wx.LI_HORIZONTAL|wx.GROW)

def HLineText(panel, text, colour='#222288'):
    """draw an Horizontal line, then SimpleText underneath
    HLineText(panel, text, **kws)
    keywords are passed to SimpleText
    """
    p = wx.Panel(panel)
    s = wx.BoxSizer(wx.HORIZONTAL)
    s.Add(wx.StaticLine(p, size=(50, 5), style=wx.LI_HORIZONTAL), 0, LEFT, 5)
    s.Add(SimpleText(p, text,  **kws),  0, LEFT, 5)
    pack(p, s)
    return p

class Check(wx.CheckBox):
    """Simple Checkbox
    c = Check(parent, default=True, label=None, **kws)
    kws passed to wx.CheckBox
    """
    def __init__(self, parent, label='', default=True, action=None, **kws):
        wx.CheckBox.__init__(self, parent, -1, label=label, **kws)
        self.SetValue({True: 1, False:0}[default])
        if action is not None:
            self.Bind(wx.EVT_CHECKBOX, action)

def MenuItem(parent, menu, label='', longtext='', action=None, kind='normal',
             checked=False):
    """Add Item to a Menu, with action
    m = Menu(parent, menu, label, longtext, action=None, kind='normal')
    """
    kinds_map = {'normal': wx.ITEM_NORMAL,
                 'radio': wx.ITEM_RADIO,
                 'check': wx.ITEM_CHECK}
    menu_kind = wx.ITEM_NORMAL
    if kind in kinds_map.values():
        menu_kind = kind
    elif kind in kinds_map:
        menu_kind = kinds_map[kind]

    item = menu.Append(-1, label, longtext, kind=menu_kind)
    if menu_kind == wx.ITEM_CHECK and checked:
        item.Check(True)

    if callable(action):
        parent.Bind(wx.EVT_MENU, action, item)
    return item


def Popup(parent, message, title, style=None, **kws):
    """Simple popup message dialog
    p = Popup(parent, message, title, **kws)
    returns output of MessageDialog.ShowModal()
    """
    if style is None:
        style = wx.OK|wx.ICON_INFORMATION
    dlg = wx.MessageDialog(parent, message, title, style=style, **kws)
    ret = dlg.ShowModal()
    dlg.Destroy()
    return ret



def ExceptionPopup(parent, title, lines, with_traceback=True,
                   style=None, **kws):
    """Modal message dialog with current Python Exception"""
    if style is None:
        style = wx.OK|wx.ICON_INFORMATION

    etype, exc, tb = sys.exc_info()
    if with_traceback:
        lines.append("Traceback (most recent calls last):")
        for tline in format_tb(tb):
            if tline.endswith('\n'): tline = tline[:-1]
            lines.append(tline)
    lines.append(f"{etype.__name__}: {exc}")

    message = '\n'.join(lines)
    dkws = {'size': (700, 350)}
    dkws.update(kws)
    dlg = ScrolledMessageDialog(parent, message, title, **dkws)
    dlg.ShowModal()
    dlg.Destroy()


def get_homedir():
    "determine home directory"
    homedir = None
    def check(method, s):
        "check that os.path.expanduser / expandvars gives a useful result"
        try:
            if method(s) not in (None, s):
                return method(s)
        except:
            pass
        return None

    # try expanding '~' -- should work on most Unixes
    homedir = check(os.path.expanduser, '~')

    # try the common environmental variables
    if homedir is  None:
        for var in ('$HOME', '$HOMEPATH', '$USERPROFILE', '$ALLUSERSPROFILE'):
            homedir = check(os.path.expandvars, var)
            if homedir is not None:
                break

    # For Windows, ask for parent of Roaming 'Application Data' directory
    if homedir is None and os.name == 'nt':
        try:
            from win32com.shell import shellcon, shell
            homedir = shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0)
        except ImportError:
            pass

    # finally, use current folder
    if homedir is None:
        homedir = os.path.abspath('.')
    return homedir


def get_cwd():
    """get current working directory
    Note: os.getcwd() can fail with permission error.

    when that happens, this changes to the users `HOME` directory
    and returns that directory so that it always returns an existing
    and readable directory.
    """
    try:
        return os.getcwd()
    except:
        curdir = os.path.abspath('.')
        os.chdir(curdir)
        return home

def gcd(parent=None, **kws):
    """Directory Browser to Change Directory"""
    if parent is None:
        parent = wx.GetApp()

    dlg = wx.DirDialog(parent, 'Choose Directory',
                       defaultPath = get_cwd(),
                       style = wx.DD_DEFAULT_STYLE)

    if dlg.ShowModal() == wx.ID_OK:
        os.chdir(dlg.GetPath())
    dlg.Destroy()
    return get_cwd()


def panel_pack(window, panel, pad=10):
    """
    a simple method to pack a single panel to a single frame
    """
    sizer = wx.BoxSizer(wx.VERTICAL)
    sizer.Add(panel, 1, wx.LEFT, 5)
    window.SetSizer(sizer)
    sizer.Fit(window)
    w0, h0 = window.GetSize()
    w1, h1 = window.GetBestSize()
    window.SetSize((max(w0, w1)+pad, max(h0, h1)+pad))

def show_wxsizes(obj):
    """recursively show sizes of wxPython objects --
    useful for avoiding size<1 errors"""
    for child in obj.GetChildren():
        try:
            csize = child.GetSize()
            if csize[0] < 1 or csize[1] < 1:
                print(child, csize)
        except:
            pass
        try:
            show_wxsizes(child)
        except:
            pass



import wx
from .text import SimpleText
from .utils import pack, LEFT

class RowPanel(wx.Panel):
    """ a simple row panel with horizontal sizer"""
    def __init__(self, parent, **kws):
        wx.Panel.__init__(self, parent, **kws)
        self.sizer = wx.BoxSizer(wx.HORIZONTAL)

    def Add(self, item, expand=0, style=LEFT, padding=2):
        self.sizer.Add(item, expand, style, padding)

    def AddText(self, label, expand=0, style=LEFT,
                padding=2, **kws):
        self.sizer.Add(SimpleText(self, label, **kws),
                       expand, style, padding)
    def pack(self):
        pack(self, self.sizer)


class GridPanel(wx.Panel):
    """A simple panel with a GridBagSizer"""

    def __init__(self, parent, nrows=10, ncols=10, pad=2, gap=5,
                 itemstyle=wx.ALIGN_CENTER, **kws):

        wx.Panel.__init__(self, parent, **kws)
        self.sizer = wx.GridBagSizer(nrows, ncols)
        self.sizer.SetVGap(gap)
        self.sizer.SetHGap(gap)

        self.irow = 0
        self.icol = 0
        self.itemstyle = itemstyle
        self.pad=pad

    def Add(self, item, irow=None, icol=None, drow=1, dcol=1, style=None,
            newrow=False, pad=None, **kws):
        """add item with default values for col, row, and size"""
        if newrow:
            self.NewRow()
        if style is None:
            style = self.itemstyle
        if irow is None:
            irow = self.irow
        if pad is None:
            pad = self.pad
        if icol is None:
            icol = self.icol
        self.sizer.Add(item, (irow, icol), (drow, dcol), style, pad, **kws)
        self.icol = self.icol + dcol

    def AddMany(self, items, newrow=False, **kws):
        """add items"""
        if newrow: self.NewRow()
        for item in items:
            self.Add(item, **kws)

    def AddManyText(self, items, newrow=False, **kws):
        """add items"""
        if newrow: self.NewRow()
        for item in items:
            self.AddText(item, **kws)

    def NewRow(self):
        "advance row, set col # = 0"
        self.irow += 1
        self.icol = 0

    def AddText(self, label, newrow=False, dcol=1, style=None, **kws):
        """add a Simple StaticText item"""
        if style is None:
            style = LEFT
        self.Add(SimpleText(self, label, style=style, **kws),
                 dcol=dcol, style=style, newrow=newrow)

    def pack(self):
        tsize = self.GetSize()
        msize = self.GetMinSize()

        self.SetSizer(self.sizer)
        self.sizer.Fit(self)
        nsize = (10*int(1.1*(max(msize[0], tsize[0])/10)),
                 10*int(1.1*(max(msize[1], tsize[1])/10.)))
        self.SetSize(nsize)



import wx

class Choice(wx.Choice):
    """Simple Choice with default and bound action
    c = Choice(panel, choices, default=0, action=None, **kws)
    """
    def __init__(self, parent, choices=None, default=0,
                 action=None, **kws):
        if choices is None:
            choices = []
        wx.Choice.__init__(self, parent, -1,  choices=choices, **kws)
        self.Select(default)
        self.Bind(wx.EVT_CHOICE, action)

    def SetChoices(self, choices):
        index = 0
        try:
            current = self.GetStringSelection()
            if current in choices:
                index = choices.index(current)
        except:
            pass
        self.Clear()
        self.AppendItems(choices)
        self.SetStringSelection(choices[index])


class YesNo(wx.Choice):
    """
    A simple wx.Choice with choices set to ('No', 'Yes')
    c = YesNo(parent, defaultyes=True, choices=('No', 'Yes'))

    has methods SetChoices(self, choices) and Select(choice)
    """
    def __init__(self, parent, defaultyes=True,
                 choices=('No', 'Yes'), size=(60, -1)):
        wx.Choice.__init__(self, parent, -1, size=size)
        self.choices = choices
        self.Clear()
        self.SetItems(self.choices)
        try:
            default = int(defaultyes)
        except:
            default = 0
        self.SetSelection(default)

    def SetChoices(self, choices):
        self.Clear()
        self.SetItems(choices)
        self.choices = choices

    def Select(self, choice):
        if isinstance(choice, int):
            self.SetSelection(0)
        elif choice in self.choices:
            self.SetSelection(self.choices.index(choice))



import os
import sys


HAS_PWD = True
try:
    import pwd
except ImportError:
    HAS_PWD = False

platform = sys.platform
if os.name == 'nt':
    platform = 'win'
if platform == 'linux2':
    platform = 'linux'

def unixpath(d):
    "unix path"
    return d.replace('\\', '/')

def winpath(d):
    "ensure path uses windows delimiters"
    if d.startswith('//'):
        d = d[1:]
    d = d.replace('/', '\\')
    return d

nativepath = unixpath
if platform == 'win':
    nativepath = winpath


def get_homedir():
    "determine home directory of current user"
    home = None
    def check(method, s):
        "check that os.path.expanduser / expandvars gives a useful result"
        try:
            if method(s) not in (None, s):
                return method(s)
        except:
            pass
        return None

    # for Unixes, allow for sudo case
    susername = os.environ.get("SUDO_USER", None)
    if home is None and HAS_PWD and susername is not None:
        home = pwd.getpwnam(susername).pw_dir

    # try expanding '~' -- should work on most Unixes
    if home is None:
        home = check(os.path.expanduser, '~')


    # try the common environmental variables
    if home is  None:
        for var in ('$HOME', '$HOMEPATH', '$USERPROFILE', '$ALLUSERSPROFILE'):
            home = check(os.path.expandvars, var)
            if home is not None:
                break

    # For Windows, ask for parent of Roaming 'Application Data' directory
    if home is None and platform == 'win':
        try:
            from win32com.shell import shellcon, shell
            home = shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0)
        except ImportError:
            pass

    # finally, use current folder
    if home is None:
        home = os.path.abspath('.')
    return nativepath(home)

def get_cwd():
    """get current working directory
    Note: os.getcwd() can fail with permission error.

    when that happens, this changes to the users `HOME` directory
    and returns that directory so that it always returns an existing
    and readable directory.
    """
    try:
        return os.getcwd()
    except:
        home = get_homedir()
        os.chdir(home)
        return home

def get_configfile(configfile):
    """get configuration file from home dir"""
    cfile = os.path.join(get_homedir(), configfile)
    if os.path.exists(cfile):
        return cfile

def save_configfile(configfile, text):
    """save text to configuration file in home dir"""
    cfile = os.path.join(get_homedir(), configfile)
    with open(cfile, 'w') as fh:
        fh.write(text)



#!/usr/bin/env python
"""
  simplified wx widgets and utilities
"""
from .version import version as __version__
__author__  = 'Matthew Newville'

import sys
import wx

if sys.platform.lower() == 'darwin':
    wx.PyApp.IsDisplayAvailable = lambda _: True

from . import utils

from .utils import (gcd, ExceptionPopup, set_sizer, pack, panel_pack,
                    show_wxsizes, SetTip, Font, HLine, Check, MenuItem, Popup,
                    RIGHT, LEFT, CEN , LCEN, RCEN, CCEN, LTEXT, FRAMESTYLE)

from .buttons import Button, ToggleButton, BitmapButton
from .choice import Choice, YesNo
from .dates import hms, DateTimeCtrl
from .dialogs import (OkCancel, FileOpen, FileSave, SelectWorkdir,
                      SavedParameterDialog, fix_filename)
from .text import SimpleText, TextCtrl, LabeledTextCtrl, HyperText
from .filechecklist import FileCheckList, FileDropTarget
from .listbox import EditableListBox
from .gridpanel import GridPanel, RowPanel
from .icons import get_icon
from .floats import (make_steps, set_float, FloatCtrl, NumericCombo,
                     FloatSpin, FloatSpinWithPin)

from .notebooks import flatnotebook
from .periodictable import  PeriodicTablePanel

from .paths import (platform, nativepath, get_homedir, get_configfile,
                    save_configfile, get_cwd)

from .colors import GUIColors, COLORS, set_color
